/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DB_TRANS_H
#define DB_TRANS_H

#if EXT3D

typedef struct 
{
  int32 sign;                 
  int32 exponent;
  int32 mantissa;
  int mantissa_bits;
} double_split;

typedef struct ThreeDVAcquisitionElement
{
  double_split element[MAX_CODEVIEW];
  int32 exponent_size;

  byte skip_flag[MAX_CODEVIEW];
  byte element_equal;
  byte exponent_skip_flag[MAX_CODEVIEW];

  int32 sign_diff[MAX_CODEVIEW];
  int32 exponent_diff[MAX_CODEVIEW];
  int32 mantissa_diff[MAX_CODEVIEW];

  byte delta_flag;
  byte element_flag;
  byte pred_mode;

  int32 precision;
  int32 mantissa_length;


  double original[MAX_CODEVIEW];
  double rec[MAX_CODEVIEW];
} ThreeDVAE;

typedef struct ThreeDVAcquisitionInfo
{
  ThreeDVAE *focal_length_x_ae;
  ThreeDVAE *focal_length_y_ae;
  ThreeDVAE *principal_point_x_ae;
  ThreeDVAE *principal_point_y_ae;
  ThreeDVAE *translation_ae;
  ThreeDVAE *depth_near_ae;
  ThreeDVAE *depth_far_ae;

#if EXT3D
  // DEPTH_PARAMETER_SET
  int num_views;
  int pred_direction;
  int pred_weight0;
  int ref_element_id0;
  int ref_element_id1;

  // NOKIA_DISP_CALC_B0150
  byte disp_param_flag;
  double d_disparity_scale[MAX_CODEVIEW][MAX_CODEVIEW];  //!< scale parameter for disparity [source_view_idx][target_view_idx]
  double d_disparity_offset[MAX_CODEVIEW][MAX_CODEVIEW];  //!< offset parameter for disparity [source_view_idx][target_view_idx]
  int i_disparity_scale[MAX_CODEVIEW][MAX_CODEVIEW];  //!< scale parameter for disparity [source_view_idx][target_view_idx]
  int i_disparity_offset[MAX_CODEVIEW][MAX_CODEVIEW];  //!< offset parameter for disparity [source_view_idx][target_view_idx]
  int i_disparity_scale_diff[MAX_CODEVIEW][MAX_CODEVIEW];  //!< scale parameter for disparity [source_view_idx][target_view_idx]
  int i_disparity_offset_diff[MAX_CODEVIEW][MAX_CODEVIEW];  //!< offset parameter for disparity [source_view_idx][target_view_idx]
#endif

} ThreeDVAcquisitionInfo;

typedef struct ThreeDVUpdateInfo 
{
  int pred_direction;
  int pred_weight0;
  int frame_num_diff0_minus1;
  int frame_num_diff1_minus1;
  int num_views_minus1;
  int camera_order[MAX_CODEVIEW];

  ThreeDVAcquisitionInfo* currr_3dv_acquisition_info;
  ThreeDVAcquisitionInfo* forward_3dv_acquisition_info;
  ThreeDVAcquisitionInfo* backward_3dv_acquisition_info;
  ThreeDVAcquisitionInfo* subsps_3dv_acquisition_info;
} ThreeDVUpdateInfo ;

//extern void GetExponentMantissa(ThreeDVAE* threeDV_ae);
extern void get_exponent_mantissa(ThreeDVAE* threeDV_ae,int voidx);
extern void get_rec_double_type(ThreeDVAE* threeDV, int voidx);
//extern int32 GetReconDoubleType(ThreeDVAE* threeDV_ae,int voidx,double* recon);
extern int  get_mem_acquisition_info(ThreeDVAcquisitionInfo** threeDV_acquisition_info);
extern void free_mem_acquisition_info(ThreeDVAcquisitionInfo* threeDV_acqusition_info);
extern void init_acquisition_info(ThreeDVAcquisitionInfo* ThreeDV_acquisition_info);
extern void copy_acquisition_info(ThreeDVAcquisitionInfo* dest_acquisition_infor, ThreeDVAcquisitionInfo* src_acquisition_info);

#if EXT3D
typedef struct ThreeDVAcquisitionElementSEI
{
  double_split element;
  int32 exponent_size;

  byte pred_mode;

  int32 precision;
  int32 mantissa_length;

  double original;
  double rec;
} ThreeDVAESEI;

typedef struct DepthAcquisitionInfoSEI
{
  ThreeDVAESEI depth_near_ae;
  ThreeDVAESEI depth_far_ae;
  ThreeDVAESEI d_min_ae;
  ThreeDVAESEI d_max_ae;
} DepthAcquisitionInfoSEI;

typedef struct MultiviewAcquisitionInfoSEI
{
  ThreeDVAESEI focal_length_x_ae;
  ThreeDVAESEI focal_length_y_ae;
  ThreeDVAESEI principal_point_x_ae;
  ThreeDVAESEI principal_point_y_ae;
  ThreeDVAESEI skew_factor_ae;
  ThreeDVAESEI rotation_ae[3][3];
  ThreeDVAESEI translation_ae[3];
} MultiviewAcquisitionInfoSEI;

extern void get_exponent_mantissa_SEI(ThreeDVAESEI* threeDV_ae);
extern void get_rec_double_type_SEI(ThreeDVAESEI* threeDV_ae);

extern void init_depth_acquisition_info_SEI(DepthAcquisitionInfoSEI* depth_acquisition_info);
extern void init_multiview_acquisition_info_SEI(MultiviewAcquisitionInfoSEI* multiview_acquisition_info);

#endif

#endif

#endif

