/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __ALC_H__
#define __ALC_H__

#include "defines.h"
#include "global.h"

#if EXT3D

#ifndef CLIP
#define CLIP(x,min,max) ( (x)<(min)?(min):((x)>(max)?(max):(x)) )
#endif

typedef imgpel XPel;

//thickness of template. Is expressed in pixels
#define MAX_TEMPLATE_SIZE                                       (1)

// max MB Size 16
#define MAX_TEMPLATE_VOLUME                                     ((MAX_TEMPLATE_SIZE * 16) * 2 + (MAX_TEMPLATE_SIZE * MAX_TEMPLATE_SIZE))

//Threshold of difference between template values in reference and decoded frames
#define REF_DEC_LUMA_DIF_THRESHOLD                (30)          // if REF_DEC_LUMA_DIF_THRESHOLD == 256, then no comparison between ref and dec to be applied at all

//  EARLY_ALC_OFF_DETECTION -this define allows to compare sums of templates and exclude ALC_Correction if templates differencies are not crucial
//  EARLY_ALC_OFF_DETECTION_THRESHOLD defines how strong ref_sum/dec_sum will be quantizaed before comparison; it uses as follows:
//  ref_sum >> = EARLY_ALC_OFF_DETECTION_THRESHOLD;
//  dec_sum >> = EARLY_ALC_OFF_DETECTION_THRESHOLD;
#define EARLY_ALC_OFF_DETECTION                                 (1)
#define EARLY_ALC_OFF_DETECTION_THRESHOLD                       (4)

// fixed point macroses
#define SINGL_MULT_FACT_PREC                                    (15)
#define MULT_FACTOR_X_255                                       (255 << SINGL_MULT_FACT_PREC)

#define ALCOK                                                   (0)

#define TM_EMPTY                                                (0)
#define TM_FILLED                                               (1)

void ALC_Create();
void ALC_Destroy();
int  ALC_GetTM(  XPel **ppRef, int RefTM_startY, int RefTM_startX, 
                XPel **ppDec, int DecTM_startY, int DecTM_startX, int Blk_height, int Blk_width);
Boolean IsTMFilled();
void    ALC_TurnOFF();
XPel**  ALC_RetTM_buf();
int**   ALC_RetTM_tempbuf();
void    PiLC_TMPelCorHist_2DArray( XPel **ppOut, int   XOut,                              int h, int w );
void    PiLC_TMPelCorHist_1DArray( XPel  *pIn,   XPel *pOut, int StrideIn, int StrideOut, int h, int w );

#endif

#endif //__ALC_H__
