/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


/*!
 ***************************************************************************
 * \file
 *    ratectl.h
 *
 * \author
 *    Zhengguo LI
 *    Athanasios Leontaris
 *
 * \date
 *    14 Jan 2003
 *
 * \brief
 *    Headerfile for rate control
 **************************************************************************
 */

#ifndef _RATE_CTL_H_
#define _RATE_CTL_H_

#include "global.h"
#include "rc_quadratic.h"


// generic functions
extern int    Qstep2QP          ( double Qstep, int qp_offset );
extern double QP2Qstep          ( int QP );
extern int    ComputeMBMAD      ( int diff[16][16] );
extern double ComputeFrameMAD   ( VideoParameters *p_Vid );
extern void   rc_store_mad      ( Macroblock *currMB );

// rate control functions
// init/copy
extern void  rc_alloc_generic           ( VideoParameters *p_Vid, RCGeneric **p_quad );
extern void  rc_free_generic            ( RCGeneric **p_quad );
extern void  rc_copy_generic            ( VideoParameters *p_Vid, RCGeneric *dst, RCGeneric *src );
extern void  rc_init_gop_params         ( VideoParameters *p_Vid, InputParameters *p_Inp );
extern void  rc_init_frame              ( VideoParameters *p_Vid, InputParameters *p_Inp);
extern void  rc_init_sequence           ( VideoParameters *p_Vid, InputParameters *p_Inp);
extern void  rc_store_slice_header_bits ( VideoParameters *p_Vid, InputParameters *p_Inp, int len);

#endif

