/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


/*!
 *************************************************************************************
 * \file vlc.h
 *
 * \brief
 *    Prototypes for VLC coding funtions
 * \author
 *     Karsten Suehring
 *************************************************************************************
 */

#ifndef _VLC_H_
#define _VLC_H_

#include "enc_statistics.h"

extern Boolean u_1  (char *tracestring, int value, Bitstream *bitstream);
extern int se_v (char *tracestring, int value, Bitstream *bitstream);
extern int ue_v (char *tracestring, int value, Bitstream *bitstream);
extern int u_v  (int n, char *tracestring, int value, Bitstream *bitstream);

extern void levrun_linfo_c2x2(int level,int run,int *len,int *info);
extern void levrun_linfo_inter(int level,int run,int *len,int *info);

extern void writeSE_Fix                  (SyntaxElement *se, Bitstream *bitstream);
extern void writeSE_UVLC                 (SyntaxElement *se, DataPartition *dp);
extern void writeSE_SVLC                 (SyntaxElement *se, DataPartition *dp);
extern void writeSE_Flag                 (SyntaxElement *se, DataPartition *dp);
extern void writeSE_invFlag              (SyntaxElement *se, DataPartition *dp);
extern void writeSE_Dummy                (SyntaxElement *se, DataPartition *dp);

extern void  writeCBP_VLC                (Macroblock* currMB, SyntaxElement *se, DataPartition *dp);
extern void  writeIntraPredMode_CAVLC    (SyntaxElement *se, DataPartition *dp);
extern int   writeSyntaxElement2Buf_UVLC (SyntaxElement *se, Bitstream* this_streamBuffer );
extern void  writeUVLC2buffer            (SyntaxElement *se, Bitstream *currStream);
extern void  writeVlcByteAlign           (VideoParameters *p_Vid, Bitstream* currStream, StatParameters *cur_stats);
extern int   writeSyntaxElement2Buf_Fixed(SyntaxElement *se, Bitstream* this_streamBuffer );
extern int   symbol2uvlc                 (SyntaxElement *se);
extern void  ue_linfo       (int n, int dummy, int *len,int *info);
extern void  se_linfo       (int mvd, int dummy, int *len,int *info);
extern void  cbp_linfo_intra_normal(int cbp, int dummy, int *len,int *info);
extern void  cbp_linfo_intra_other (int cbp, int dummy, int *len,int *info);
extern void  cbp_linfo_inter_normal(int cbp, int dummy, int *len,int *info);
extern void  cbp_linfo_inter_other (int cbp, int dummy, int *len,int *info);

extern int   writeSyntaxElement_VLC(SyntaxElement *se, DataPartition *this_dataPart);
extern int   writeSyntaxElement_TotalZeros(SyntaxElement *se, DataPartition *this_dataPart);
extern int   writeSyntaxElement_TotalZerosChromaDC(VideoParameters *p_Vid, SyntaxElement *se, DataPartition *this_dataPart);
extern int   writeSyntaxElement_Run(SyntaxElement *se, DataPartition *this_dataPart);
extern int   writeSyntaxElement_NumCoeffTrailingOnes(SyntaxElement *se, DataPartition *this_dataPart);
extern int   writeSyntaxElement_NumCoeffTrailingOnesChromaDC(VideoParameters *p_Vid, SyntaxElement *se, DataPartition *this_dataPart);
extern int   writeSyntaxElement_Level_VLC1(SyntaxElement *se, DataPartition *this_dataPart, int profile_idc);
extern int   writeSyntaxElement_Level_VLCN(SyntaxElement *se, int vlc, DataPartition *this_dataPart, int profile_idc);

extern void writeUVLC_CAVLC (Macroblock *currMB, SyntaxElement *se, DataPartition *dp);
extern void writeSVLC_CAVLC (Macroblock *currMB, SyntaxElement *se, DataPartition *dp);
extern void writeFlag_CAVLC (Macroblock *currMB, SyntaxElement *se, DataPartition *dp);

extern void reset_mb_nz_coeff(VideoParameters *p_Vid, int mb_number); 

#endif

