/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 ***************************************************************************
 * \file
 *    wp.h
 *
 * \author
 *    Alexis Michael Tourapis
 *
 * \date
 *    22. February 2008
 *
 * \brief
 *    Headerfile for weighted prediction support
 **************************************************************************
 */

#ifndef _WP_H_
#define _WP_H_

#include "wp_lms.h"
#include "wp_mcprec.h"
#include "wp_mciter.h"

#define DEBUG_WP  0

void InitWP              (VideoParameters *p_Vid, InputParameters *p_Inp, int force_wp_method);
void ResetWP             (VideoParameters *p_Vid, InputParameters *p_Inp);

extern void   EstimateWPBSliceAlg0   (Slice *currSlice);
extern void   EstimateWPPSliceAlg0   (Slice *currSlice, int offset);
extern int    TestWPPSliceAlg0       (Slice *currSlice, int offset);
extern int    TestWPBSliceAlg0       (Slice *currSlice, int method);
extern double ComputeImgSum          (imgpel **CurrentImage, int height, int width);
extern void   ComputeImgSumBlockBased(imgpel **CurrentImage, int height_in_blk, int width_in_blk, int blk_size_y, int blk_size_x, int start_blk, int end_blk, double *dc);
extern int64  ComputeSumBlockBased   (imgpel **CurrentImage, int height_in_blk, int width_in_blk, int blk_size_y, int blk_size_x, int start_blk, int end_blk);

extern void   ComputeImplicitWeights    (Slice *currSlice,
                                         short default_weight[3],
                                         short im_weight[6][MAX_REFERENCE_PICTURES][MAX_REFERENCE_PICTURES][3]);
extern void   ComputeExplicitWPParamsLMS(Slice *currSlice,
                                         int select_offset,
                                         int start_mb,
                                         int end_mb, 
                                         short default_weight[3],
                                         short weight[6][MAX_REFERENCE_PICTURES][3],
                                         short offset[6][MAX_REFERENCE_PICTURES][3]);
extern void   ComputeExplicitWPParamsJNT(Slice *currSlice,
                                         int start_mb,
                                         int end_mb, 
                                         short default_weight[3],
                                         short weight[6][MAX_REFERENCE_PICTURES][3],
                                         short offset[6][MAX_REFERENCE_PICTURES][3]);
#if EXT3D
extern void   ComputeExplicitWPParamsDRB(Slice* currSlice,short default_weight[3],
                                         short weight[6][MAX_REFERENCE_PICTURES][3],
                                         short offset[6][MAX_REFERENCE_PICTURES][3]);
extern void   EstimateWPPSliceAlg3(Slice* currSlice,int select_offset);
extern void   EstimateWPBSliceAlg3(Slice* currSlice);
#endif


#endif

