/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 ***************************************************************************
 * \file
 *    wp_mcprec.h
 *
 * \brief
 *    Headerfile for Improved Motion Compensatation Precision Scheme using Weighted Prediction
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *     - Athanasios Leontaris            <aleon@dolby.com>
 *
 * \date
 *    16 July 2008
 *
 **************************************************************************
 */

#ifndef _WP_MCPREC_H_
#define _WP_MCPREC_H_

typedef struct
{
  int PicNum;              // PicNum/FrameNum
  int POCNum;              // POC
}
WeightedPredRefX;

typedef struct
{
  int algorithm;
}
WPXPass;

typedef struct wpx_object
{
  int               num_wp_ref_list[2];         // num of elements in each of the above matrices [LIST]
  WeightedPredRefX *wp_ref_list[2];             // structure with reordering and WP information for ref frames [LIST]
  WPXPass          *curr_wp_rd_pass;
  WPXPass           wp_rd_passes[3];            // frame_picture [0...4] (MultiRefWeightedPred == 2)
}
WPXObject;

extern void   wpxInitWPXObject( VideoParameters *p_Vid );
extern void   wpxFreeWPXObject( VideoParameters *p_Vid );
extern void   wpxInitWPXPasses( VideoParameters *p_Vid, InputParameters *p_Inp );
extern void   wpxModifyRefPicList( Slice *currSlice );
// Note that at some point, InputParameters p_Inp contents should be copied into VideoParameters *p_Vid. 
// This would eliminate need of having to use both structures
extern int    wpxDetermineWP( Slice *currSlice, int clist, int n );
extern void   wpxAdaptRefNum( Slice *currSlice );

#endif
