/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2014, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


/*!
 *************************************************************************************
 * \file quantChroma.c
 *
 * \brief
 *    Quantization initialization function for Chroma blocks
 *
 * \author
 *    Main contributors (see contributors.h for copyright, address and affiliation details)
 *    - Alexis Michael Tourapis                  <alexismt@ieee.org>
 *
 *************************************************************************************
 */

#include "contributors.h"

#include "global.h"
#include "quant4x4.h"
#include "quantChroma.h"

/*!
************************************************************************
* \brief
*    Quantization initialization function
*
************************************************************************
*/
void init_quant_Chroma(Slice *currSlice)
{
  VideoParameters *p_Vid = currSlice->p_Vid;
  InputParameters *p_Inp = currSlice->p_Inp;

  if (p_Inp->UseRDOQuant == 1 && p_Inp->RDOQ_CR == 1)
  {
    currSlice->quant_ac4x4cr = quant_ac4x4_trellis;
    if (p_Inp->RDOQ_DC_CR)
    {
      if (p_Vid->yuv_format == YUV422)
        currSlice->quant_dc_cr = quant_dc4x2_trellis;
      else
        currSlice->quant_dc_cr = quant_dc2x2_trellis;
    }
    else
    {
      if (p_Vid->yuv_format == YUV422)
        currSlice->quant_dc_cr   = quant_dc4x2_normal;
      else
        currSlice->quant_dc_cr   = quant_dc2x2_normal;
    }
    if (currSlice->symbol_mode == CABAC)
      currSlice->rdoq_dc_cr = rdoq_dc_cr_CABAC;
    else
      currSlice->rdoq_dc_cr = rdoq_dc_cr_CAVLC;
  }
  else if (p_Inp->UseRDOQuant == 1 || (!(currSlice->p_Vid)->AdaptiveRounding))
  {
    currSlice->quant_ac4x4cr = quant_ac4x4_normal;
    if (p_Vid->yuv_format == YUV422)
      currSlice->quant_dc_cr   = quant_dc4x2_normal;
    else
      currSlice->quant_dc_cr   = quant_dc2x2_normal;
  }
  else
  {
    currSlice->quant_ac4x4cr = quant_ac4x4_around;
    if (p_Vid->yuv_format == YUV422)
      currSlice->quant_dc_cr   = quant_dc4x2_around;
    else
      currSlice->quant_dc_cr   = quant_dc2x2_around;
  }
}


